-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.relevantDevice = "differential_R"
M.inputValues = nil
local differential = nil

--local origViscousCoef = 0
local inputAVSmoother = newExponentialSmoothing(50)
local outputAVSmoother = newExponentialSmoothing(50)

local function updateGFX()

  local inputAV = inputAVSmoother:get(differential.inputAV)
  local outputAV1 = outputAVSmoother:get(differential.outputAV1) --left wheel
  local outputAV2 = outputAVSmoother:get(differential.outputAV2) --right wheel
  local throttle = electrics.values['throttle_input']
  local brake = electrics.values['brake_input']
  local steer = electrics.values['steering_input']  --positive is right turn
  local absSteer = math.abs(steer)
  local lateralG = sensors.gx2  --negative is right turn
  local absLateralG = math.abs(lateralG)
  local steerDirectionCoef = clamp(steer * 10,-1,1) --positive is right turn
  local sideAccelDirectionCoef = clamp(- lateralG * 1,-1,1) --positive is right turn
  
  --differential.lsdLockCoef = 0.02 + 0.18 * clamp(throttle * 10,0,1)
  differential.lsdLockCoef = 0.02 + 0.38 * clamp((outputAV2 - outputAV1) * 2 * sideAccelDirectionCoef * clamp(throttle * 5 - 1,0,1),0,1) --active locking

  differential.diffTorqueSplitA = clamp(((steer - lateralG/11) * 0.5 * clamp(throttle * 5,0,1) + 0.5), 0.25, 0.75) --torque vectoring
  --differential.diffTorqueSplitA = 0.5  --left wheel

  --differential.lsdPreload = 5 + 0.45 * clamp(throttle * 5,0,1)
  --differential.lsdPreload = 5 + 2500 * clamp(math.abs(outputAV2 - outputAV1)*0.5,0,1)
  differential.lsdPreload = 5 + 45 * clamp((throttle - 0.2) * 10,0,1) + 1000 * clamp(math.abs(outputAV2 - outputAV1) * 0.5 - 0.3,0,1) --passive locking
  --differential.lsdPreload = 0
  
  differential.diffTorqueSplitB = 1 - differential.diffTorqueSplitA --right wheel
  differential.lsdRevLockCoef = differential.lsdLockCoef
  
end

local function init(jbeamData)
  differential = powertrain.getDevice(jbeamData.transfercaseName)

  if differential then
  differential.lsdLockCoef = 0.1
  differential.lsdRevLockCoef = 0.1
  differential.lsdPreLoad = 10
  differential.diffTorqueSplit = 0.5

    --origViscousCoef = transfercase.viscousCoef
    M.updateGFX = updateGFX
  end
end

M.init = init
M.reset = nop
M.updateGFX = nop

return M